@@ -17,16 +17,237 @@ limitations under the License.
 local logger = require "rspamd_logger"
 local lutil = require "lua_util"
 local rspamd_util = require "rspamd_util"
+local ts = require("tableshape").types
 
 local exports = {}
 
 local E = {}
+local N = "lua_redis"
+
+local common_schema = ts.shape {
+  timeout = (ts.number + ts.string / lutil.parse_time_interval):is_optional(),
+  db = ts.string:is_optional(),
+  database = ts.string:is_optional(),
+  dbname = ts.string:is_optional(),
+  prefix = ts.string:is_optional(),
+  password = ts.string:is_optional(),
+  expand_keys = ts.boolean:is_optional(),
+  sentinels = (ts.string + ts.array_of(ts.string)):is_optional(),
+  sentinel_watch_time = (ts.number + ts.string / lutil.parse_time_interval):is_optional(),
+  sentinel_masters_pattern = ts.string:is_optional(),
+}
+
+local config_schema =
+  ts.shape({
+    read_servers = ts.string + ts.array_of(ts.string),
+    write_servers = ts.string + ts.array_of(ts.string),
+  }, {extra_opts = common_schema}) +
+  ts.shape({
+    servers = ts.string + ts.array_of(ts.string),
+  }, {extra_opts = common_schema}) +
+  ts.shape({
+    server = ts.string + ts.array_of(ts.string),
+  }, {extra_opts = common_schema})
+
+exports.config_schema = config_schema
+
+
+local function redis_query_sentinel(ev_base, params, initialised)
+  local function flatten_redis_table(tbl)
+    local res = {}
+    for i=1,#tbl,2 do
+      res[tbl[i]] = tbl[i + 1]
+    end
+
+    return res
+  end
+  -- Coroutines syntax
+  local rspamd_redis = require "rspamd_redis"
+  local addr = params.sentinels:get_upstream_round_robin()
+
+  local is_ok, connection = rspamd_redis.connect_sync({
+    host = addr:get_addr(),
+    timeout = params.timeout,
+    config = rspamd_config,
+    ev_base = ev_base,
+  })
+
+  if not is_ok then
+    logger.errx(rspamd_config, 'cannot connect sentinel at address: %s',
+        tostring(addr:get_addr()))
+    addr:fail()
+
+    return
+  end
+
+  -- Get masters list
+  connection:add_cmd('SENTINEL', {'masters'})
+
+  local ok,result = connection:exec()
+
+  if ok and result and type(result) == 'table' then
+    local masters = {}
+    for _,m in ipairs(result) do
+      local master = flatten_redis_table(m)
+
+      if params.sentinel_masters_pattern then
+        if master.name:match(params.sentinel_masters_pattern) then
+          lutil.debugm(N, 'found master %s with ip %s and port %s',
+              master.name, master.ip, master.port)
+          masters[master.name] = master
+        else
+          lutil.debugm(N, 'skip master %s with ip %s and port %s, pattern %s',
+              master.name, master.ip, master.port, params.sentinel_masters_pattern)
+        end
+      else
+        lutil.debugm(N, 'found master %s with ip %s and port %s',
+            master.name, master.ip, master.port)
+        masters[master.name] = master
+      end
+    end
+
+    -- For each master we need to get a list of slaves
+    for k,v in pairs(masters) do
+      v.slaves = {}
+      local slave_result
+
+      connection:add_cmd('SENTINEL', {'slaves', k})
+      ok,slave_result = connection:exec()
+
+      if ok then
+        for _,s in ipairs(slave_result) do
+          local slave = flatten_redis_table(s)
+          lutil.debugm(N, rspamd_config,
+              'found slave form master %s with ip %s and port %s',
+              v.name, slave.ip, slave.port)
+          v.slaves[#v.slaves + 1] = slave
+        end
+      end
+    end
+
+    -- We now form new strings for masters and slaves
+    local read_servers_tbl, write_servers_tbl = {}, {}
+
+    for _,master in pairs(masters) do
+      write_servers_tbl[#write_servers_tbl + 1] = string.format(
+          '%s:%s', master.ip, master.port
+      )
+      read_servers_tbl[#read_servers_tbl + 1] = string.format(
+          '%s:%s', master.ip, master.port
+      )
+
+      for _,slave in ipairs(master.slaves) do
+        read_servers_tbl[#read_servers_tbl + 1] = string.format(
+            '%s:%s', slave.ip, slave.port
+        )
+      end
+    end
+
+    local read_servers_str = table.concat(read_servers_tbl, ',')
+    local write_servers_str = table.concat(read_servers_tbl, ',')
+
+    lutil.debugm(N, rspamd_config,
+        'new servers list: %s read; %s write', read_servers_str, write_servers_str)
+
+    if read_servers_str ~= params.read_servers_str then
+      local upstream_list = require "rspamd_upstream_list"
+
+      local read_upstreams = upstream_list.create(rspamd_config,
+          read_servers_str, 6379)
+
+      if read_upstreams then
+        logger.infox(rspamd_config, 'sentinel %s: replace read servers with new list: %s',
+            addr:get_addr():to_string(true), read_servers_str)
+        params.read_servers = read_upstreams
+        params.read_servers_str = read_servers_str
+      end
+    end
+
+    if write_servers_str ~= params.write_servers_str then
+      local upstream_list = require "rspamd_upstream_list"
+
+      local write_upstreams = upstream_list.create(rspamd_config,
+          write_servers_str, 6379)
+
+      if write_upstreams then
+        logger.infox(rspamd_config, 'sentinel %s: replace write servers with new list: %s',
+            addr:get_addr():to_string(true), write_servers_str)
+        params.write_servers = write_upstreams
+        params.write_servers_str = write_servers_str
+      end
+    end
+
+    addr:ok()
+  else
+    logger.errx('cannot get data from Redis Sentinel %s: %s',
+        addr:get_addr():to_string(true), result)
+    addr:fail()
+  end
+
+end
+
+local function add_redis_sentinels(params)
+  local upstream_list = require "rspamd_upstream_list"
+
+  local upstreams_sentinels = upstream_list.create(rspamd_config,
+      params.sentinels, 5000)
+
+  if not upstreams_sentinels then
+    logger.errx(rspamd_config, 'cannot load redis sentinels string: %s',
+        params.sentinels)
+
+    return
+  end
+
+  params.sentinels = upstreams_sentinels
+
+  if not params.sentinel_watch_time then
+    params.sentinel_watch_time = 60 -- Each minute
+  end
+
+  rspamd_config:add_on_load(function(cfg, ev_base, worker)
+    local initialised = false
+    if worker:is_scanner() then
+      rspamd_config:add_periodic(ev_base, 0.0, function()
+        redis_query_sentinel(ev_base, params, initialised)
+        initialised = true
+
+        return params.sentinel_watch_time
+      end, false)
+    end
+  end)
+end
+
+local cached_results = {}
+
+local function calculate_redis_hash(params)
+  local cr = require "rspamd_cryptobox_hash"
+
+  local h = cr.create()
+
+  local function rec_hash(k, v)
+    if type(v) == 'string' then
+      h:update(k)
+      h:update(v)
+    elseif type(v) == 'number' then
+      h:update(k)
+      h:update(tostring(v))
+    elseif type(v) == 'table' then
+      for kk,vv in pairs(v) do
+        rec_hash(kk, vv)
+      end
+    end
+  end
+
+  rec_hash('top', params)
+
+  return h:base32()
+end
 
 --[[[
 -- @module lua_redis
 -- This module contains helper functions for working with Redis
 --]]
-
 local function try_load_redis_servers(options, rspamd_config, result)
   local default_port = 6379
   local default_timeout = 1.0
@@ -45,6 +266,8 @@ local function try_load_redis_servers(options, rspamd_config, result)
       upstreams_read = upstream_list.create(options['read_servers'],
         default_port)
     end
+
+    result.read_servers_str = options['read_servers']
   elseif options['servers'] then
     if rspamd_config then
       upstreams_read = upstream_list.create(rspamd_config,
@@ -52,6 +275,8 @@ local function try_load_redis_servers(options, rspamd_config, result)
     else
       upstreams_read = upstream_list.create(options['servers'], default_port)
     end
+
+    result.read_servers_str = options['servers']
     read_only = false
   elseif options['server'] then
     if rspamd_config then
@@ -60,6 +285,8 @@ local function try_load_redis_servers(options, rspamd_config, result)
     else
       upstreams_read = upstream_list.create(options['server'], default_port)
     end
+
+    result.read_servers_str = options['server']
     read_only = false
   end
 
@@ -72,9 +299,11 @@ local function try_load_redis_servers(options, rspamd_config, result)
         upstreams_write = upstream_list.create(options['write_servers'],
                 default_port)
       end
+      result.write_servers_str = options['write_servers']
       read_only = false
     elseif not read_only then
       upstreams_write = upstreams_read
+      result.write_servers_str = result.read_servers_str
     end
   end
 
@@ -110,20 +339,45 @@ local function try_load_redis_servers(options, rspamd_config, result)
     result['password'] = options['password']
   end
 
-  if read_only and not result.write_servers then
+  if read_only and not upstreams_write then
     result.read_only = true
-  elseif result.write_servers then
+  elseif upstreams_write then
     result.read_only = false
   end
 
   if upstreams_read then
     result.read_servers = upstreams_read
+
     if upstreams_write then
       result.write_servers = upstreams_write
     end
+
+    local h = calculate_redis_hash(result)
+
+    if cached_results[h] then
+      for k,v in pairs(cached_results[h]) do
+        result[k] = v
+      end
+      lutil.debugm(N, 'reused redis server: %s', result)
+      return true
+    end
+
+    result.hash = h
+    cached_results[h] = result
+
+    if not result.read_only and options.sentinels then
+      result.sentinels = options.sentinels
+      add_redis_sentinels(result)
+    end
+
+    lutil.debugm(N, 'loaded redis server: %s', result)
     return true
   end
 
+  lutil.debugm(N, rspamd_config,
+      'cannot load redis server from obj: %s, processed to %s',
+      options, result)
+
   return false
 end
 
@@ -137,6 +391,7 @@ local function rspamd_parse_redis_server(module_name, module_opts, no_fallback)
 
   -- Try local options
   local opts
+  lutil.debugm(N, rspamd_config, 'try load redis config for: %s', module_name)
   if not module_opts then
     opts = rspamd_config:get_all_opt(module_name)
   else
@@ -161,7 +416,12 @@ local function rspamd_parse_redis_server(module_name, module_opts, no_fallback)
     end
   end
 
-  if no_fallback then return nil end
+  if no_fallback then
+    logger.infox(rspamd_config, "cannot find Redis definitions for %s and fallback is disabled",
+        module_name)
+
+    return nil
+  end
 
   -- Try global options
   opts = rspamd_config:get_all_opt('redis')
@@ -171,6 +431,7 @@ local function rspamd_parse_redis_server(module_name, module_opts, no_fallback)
 
     if opts[module_name] then
       ret = try_load_redis_servers(opts[module_name], rspamd_config, result)
+
       if ret then
         return result
       end
@@ -190,17 +451,18 @@ local function rspamd_parse_redis_server(module_name, module_opts, no_fallback)
       end
 
       if ret then
-        logger.infox(rspamd_config, "using default redis server for module %s",
-          module_name)
+        logger.infox(rspamd_config, "use default Redis settings for %s",
+            module_name)
+        return result
       end
     end
   end
 
   if result.read_servers then
-    return result
-  else
-    return nil
+      return result
   end
+
+  return nil
 end
 
 --[[[
@@ -589,6 +851,10 @@ local function rspamd_redis_make_request(task, redis_params, key, is_write,
     options['dbname'] = redis_params['db']
   end
 
+  lutil.debugm(N, task, 'perform request to redis server' ..
+      ' (host=%s, timeout=%s): cmd: %s, arguments: %s', ip_addr,
+      options.timeout, options.cmd, args)
+
   local ret,conn = rspamd_redis.make_request(options)
 
   if not ret then
@@ -674,6 +940,9 @@ local function redis_make_request_taskless(ev_base, cfg, redis_params, key,
     options['dbname'] = redis_params['db']
   end
 
+  lutil.debugm(N, cfg, 'perform taskless request to redis server' ..
+      ' (host=%s, timeout=%s): cmd: %s, arguments: %s', options.host,
+      options.timeout, options.cmd, args)
   local ret,conn = rspamd_redis.make_request(options)
   if not ret then
     logger.errx('cannot execute redis request')
@@ -961,7 +1230,7 @@ end
 
 exports.exec_redis_script = exec_redis_script
 
-local function redis_connect_sync(redis_params, is_write, key, cfg)
+local function redis_connect_sync(redis_params, is_write, key, cfg, ev_base)
   if not redis_params then
     return false,nil
   end
@@ -990,8 +1259,29 @@ local function redis_connect_sync(redis_params, is_write, key, cfg)
   local options = {
     host = addr:get_addr(),
     timeout = redis_params['timeout'],
+    config = cfg or rspamd_config,
+    ev_base = ev_base or rspamadm_ev_base,
+    session = redis_params.session or rspamadm_session
   }
 
+  for k,v in pairs(redis_params) do
+    options[k] = v
+  end
+
+  if not options.config then
+    logger.errx('config is not set')
+    return false,nil,addr
+  end
+
+  if not options.ev_base then
+    logger.errx('ev_base is not set')
+    return false,nil,addr
+  end
+
+  if not options.session then
+    logger.errx('session is not set')
+    return false,nil,addr
+  end
 
   local ret,conn = rspamd_redis.connect_sync(options)
   if not ret then
@@ -1018,4 +1308,209 @@ end
 
 exports.redis_connect_sync = redis_connect_sync
 
+--[[[
+-- @function lua_redis.request(redis_params, attrs, req)
+-- Sends a request to Redis synchronously with coroutines or asynchronously using
+-- a callback (modern API)
+-- @param redis_params a table of redis server parameters
+-- @param attrs a table of redis request attributes (e.g. task, or ev_base + cfg + session)
+-- @param req a table of request: a command + command options
+-- @return {result,data/connection,address} boolean result, connection object in case of async request and results if using coroutines, redis server address
+--]]
+
+exports.request = function(redis_params, attrs, req)
+  local lua_util = require "lua_util"
+
+  if not attrs or not redis_params or not req then
+    logger.errx('invalid arguments for redis request')
+    return false,nil,nil
+  end
+
+  if not (attrs.task or (attrs.config and attrs.ev_base)) then
+    logger.errx('invalid attributes for redis request')
+    return false,nil,nil
+  end
+
+  local opts = lua_util.shallowcopy(attrs)
+
+  local log_obj = opts.task or opts.config
+
+  local addr
+
+  if opts.callback then
+    -- Wrap callback
+    local callback = opts.callback
+    local function rspamd_redis_make_request_cb(err, data)
+      if err then
+        addr:fail()
+      else
+        addr:ok()
+      end
+      callback(err, data, addr)
+    end
+    opts.callback = rspamd_redis_make_request_cb
+  end
+
+  local rspamd_redis = require "rspamd_redis"
+  local is_write = opts.is_write
+
+  if opts.key then
+    if is_write then
+      addr = redis_params['write_servers']:get_upstream_by_hash(attrs.key)
+    else
+      addr = redis_params['read_servers']:get_upstream_by_hash(attrs.key)
+    end
+  else
+    if is_write then
+      addr = redis_params['write_servers']:get_upstream_master_slave(attrs.key)
+    else
+      addr = redis_params['read_servers']:get_upstream_round_robin(attrs.key)
+    end
+  end
+
+  if not addr then
+    logger.errx(log_obj, 'cannot select server to make redis request')
+  end
+
+  opts.host = addr:get_addr()
+  opts.timeout = redis_params.timeout
+
+  if type(req) == 'string' then
+    opts.cmd = req
+  else
+    -- XXX: modifies the input table
+    opts.cmd = table.remove(req, 1);
+    opts.args = req
+  end
+
+  if redis_params.password then
+    opts.password = redis_params.password
+  end
+
+  if redis_params.db then
+    opts.dbname = redis_params.db
+  end
+
+  lutil.debugm(N, 'perform generic request to redis server' ..
+      ' (host=%s, timeout=%s): cmd: %s, arguments: %s', addr,
+      opts.timeout, opts.cmd, opts.args)
+
+  if opts.callback then
+    local ret,conn = rspamd_redis.make_request(opts)
+    if not ret then
+      logger.errx(log_obj, 'cannot execute redis request')
+      addr:fail()
+    end
+
+    return ret,conn,addr
+  else
+    -- Coroutines version
+    local ret,conn = rspamd_redis.connect_sync(opts)
+    if not ret then
+      logger.errx(log_obj, 'cannot execute redis request')
+      addr:fail()
+    else
+      conn:add_cmd(opts.cmd, opts.args)
+      return conn:exec()
+    end
+    return false,nil,addr
+  end
+end
+
+--[[[
+-- @function lua_redis.connect(redis_params, attrs)
+-- Connects to Redis synchronously with coroutines or asynchronously using a callback (modern API)
+-- @param redis_params a table of redis server parameters
+-- @param attrs a table of redis request attributes (e.g. task, or ev_base + cfg + session)
+-- @return {result,connection,address} boolean result, connection object, redis server address
+--]]
+
+exports.connect = function(redis_params, attrs)
+  local lua_util = require "lua_util"
+
+  if not attrs or not redis_params then
+    logger.errx('invalid arguments for redis connect')
+    return false,nil,nil
+  end
+
+  if not (attrs.task or (attrs.config and attrs.ev_base)) then
+    logger.errx('invalid attributes for redis connect')
+    return false,nil,nil
+  end
+
+  local opts = lua_util.shallowcopy(attrs)
+
+  local log_obj = opts.task or opts.config
+
+  local addr
+
+  if opts.callback then
+    -- Wrap callback
+    local callback = opts.callback
+    local function rspamd_redis_make_request_cb(err, data)
+      if err then
+        addr:fail()
+      else
+        addr:ok()
+      end
+      callback(err, data, addr)
+    end
+    opts.callback = rspamd_redis_make_request_cb
+  end
+
+  local rspamd_redis = require "rspamd_redis"
+  local is_write = opts.is_write
+
+  if opts.key then
+    if is_write then
+      addr = redis_params['write_servers']:get_upstream_by_hash(attrs.key)
+    else
+      addr = redis_params['read_servers']:get_upstream_by_hash(attrs.key)
+    end
+  else
+    if is_write then
+      addr = redis_params['write_servers']:get_upstream_master_slave(attrs.key)
+    else
+      addr = redis_params['read_servers']:get_upstream_round_robin(attrs.key)
+    end
+  end
+
+  if not addr then
+    logger.errx(log_obj, 'cannot select server to make redis connect')
+  end
+
+  opts.host = addr:get_addr()
+  opts.timeout = redis_params.timeout
+
+  if redis_params.password then
+    opts.password = redis_params.password
+  end
+
+  if redis_params.db then
+    opts.dbname = redis_params.db
+  end
+
+  if opts.callback then
+    local ret,conn = rspamd_redis.connect(opts)
+    if not ret then
+      logger.errx(log_obj, 'cannot execute redis connect')
+      addr:fail()
+    end
+
+    return ret,conn,addr
+  else
+    -- Coroutines version
+    local ret,conn = rspamd_redis.connect_sync(opts)
+    if not ret then
+      logger.errx(log_obj, 'cannot execute redis connect')
+      addr:fail()
+    else
+      return true,conn,addr
+    end
+
+    return false,nil,addr
+  end
+end
+
+
 return exports
